# Some code taken from:
# Revelation 0.4.5 - a password manager for GNOME 2
# http://oss.codepoet.no/revelation/
# $Id: config.py.in 412 2005-08-09 10:05:16Z erikg $
#
# module for configuration handling
#
#
# Copyright (c) 2003-2005 Erik Grinaker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


import utilRevelation

import gconf, dbus.glib, os, commands
import locale

APPNAME		= "Forcefield"
PACKAGE		= "@PACKAGE@"
VERSION		= "@VERSION@"
DATAVERSION	= 1
RELNAME		= _("Safe the bits!")

URL		= "http://www.bockcay.de/forcefield"
AUTHORS		= [ "Oliver Ripka <forcefield@bockcay.de>" ]
ARTISTS		= [ "Oliver Ripka <forcefield@bockcay.de>" ]

COPYRIGHT	= "Copyright 2006-2007 Oliver Ripka"

DIR_GCONFSCHEMAS= "@pkgschemadir@"
DIR_ICONS	= "@datadir@/icons"
DIR_UI		= "@pkgdatadir@/ui"
DIR_PATCHES = "@pkgdatadir@/patches"
FILE_GCONFTOOL	= "@GCONFTOOL@"

DISMOUNT_ON_SUSPEND_PATCH = DIR_PATCHES+"/sleep.patch"
NOT_DISMOUNT_ON_SUSPEND_PATCH= DIR_PATCHES+"/sleep.unpatch"
ACPI_SLEEP_HOOK_SCRIPT= "/etc/acpi/sleep.sh"

SUDOERS_PATCH = DIR_PATCHES+"/sudoers.patch"
SUDOERS_UNPATCH= DIR_PATCHES+"/sudoers.unpatch"
SUDOERS_FILE= "/etc/sudoers"

SCHEMA_FILE = "forcefield.schemas"

"THESE ARE THE SAME AS DEFINED IN THE SCHEMA FILE"
MOUNT_HISTORY = "mount_history"
DEV_HISTORY   = "dev_history"
OPTIONS_HISTORY= "options_history"
CACHE_HISTORY = "cache_history"
COMMAND		  = "command"
BACKGROUND	= "background_start"

DISMOUNT_ON_SUSPEND = "dismount_on_suspend"
DISMOUNT_ON_SCREENSAVER = "dismount_on_screensaver"
DISMOUNT_ON_LOGOFF = "dismount_on_logoff"
AFTER_MOUNT_COMMAND = "after_mount_command"

SUPPRESS_SUID_WARNING = "suppressSuidWarning"

OPTIONS= [MOUNT_HISTORY, DEV_HISTORY, CACHE_HISTORY, COMMAND, BACKGROUND, SUPPRESS_SUID_WARNING]

DEBUG = False
MOUNT_FILESYSTEMS = ['auto', 'ext3', 'ext2', 'vfat', 'xfs']
output = commands.getoutput("whereis -b ntfs-3g").split()
try:
	if output[1] != "": 
			MOUNT_FILESYSTEMS.append('ntfs-3g')
except:
	"no ntfs-3f"

LICENSE		= ("""Forcefield is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; either version 2 of the License, or 
(at your option) any later version.

Forcefield is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with Revelation; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA""")


"IF THE SIZE OF A PARTITIO IS UNDER THESE VALUES, IT IS CONSIDERED AS EXTENDED PARTITION"
EXTENDED_LIMIT_SIZE   = 1024
EXTENDED_LIMIT_BLOCKS = 2
class ConfigError(Exception):
	"Configuration error exception"
	pass
	
class Config(object):
	"Configuration class"
	def __init__(self, basedir = "/apps/forcefield"):
		
		self.basedir	= basedir
		self.client	= gconf.client_get_default()
		self.callbacks	= {}

		self.client.add_dir(self.basedir, gconf.CLIENT_PRELOAD_NONE)
		
	def __cb_notify(self, client, id, entry, data):
		"Callback for handling gconf notifications"

		value = entry.get_value()

		if value == None:
			return

		elif value.type == gconf.VALUE_STRING:
			v = value.get_string()

		elif value.type == gconf.VALUE_BOOL:
			v = value.get_bool()

		elif value.type == gconf.VALUE_INT:
			v = value.get_int()


		# look up and call the callback
		if self.callbacks.has_key(id) == False:

			# workaround for 64-bit overflows in gnome-python (bug #170822)
			id += (2 ** 32)

			if self.callbacks.has_key(id) == False:
				raise ConfigError

		callback, userdata = self.callbacks[id]
		callback(entry.get_key(), v, userdata)


	def __resolve_keypath(self, key):
		"Resolves a key path"

		return key[0] == "/" and key or self.basedir + "/" + key


	def forget(self, id):
		"Forgets a monitored key"

		if not self.callbacks.has_key(id):
			raise ConfigError

		self.client.notify_remove(id)
		del self.callbacks[id]


	def get(self, key):
		"Looks up a config value"

		value = self.client.get(self.__resolve_keypath(key))

		if value is None:
			raise ConfigError

		elif value.type == gconf.VALUE_STRING:
			return str(value.get_string())

		elif value.type == gconf.VALUE_INT:
			return value.get_int()

		elif value.type == gconf.VALUE_BOOL:
			return value.get_bool()

		elif value.type == gconf.VALUE_LIST:
			return self.client.get_list(self.__resolve_keypath(key), gconf.VALUE_STRING)

	def monitor(self, key, callback, userdata = None):
		"Monitor a config key for changes"

		key = self.__resolve_keypath(key)

		id = self.client.notify_add(key, self.__cb_notify)
		self.callbacks[id] = ( callback, userdata )

		# call the callback to set an initial state
		callback(key, self.get(key), userdata)

		return id
	

	def addToList(self, key, value):
		"Adds a value to a list if it isnt in the list yet"
		nodes = self.get(key)
		
		isunique = True

		for x in nodes:
			if x == value:
				isunique = False
				
		if isunique:
			nodes = nodes + [value]
			self.client.set_list(self.__resolve_keypath(key), gconf.VALUE_STRING, nodes )

	def set(self, key, value):
		"Sets a configuration value"

		node = self.client.get(self.__resolve_keypath(key))

		if node is None:
			raise ConfigError

		elif node.type == gconf.VALUE_STRING:
			node.set_string(value)

		elif node.type == gconf.VALUE_BOOL:
			node.set_bool(value)

		elif node.type == gconf.VALUE_INT:
			node.set_int(int(value))

		self.client.set(self.__resolve_keypath(key), node)


	def set_force(self, key, value):
		"Sets a value directly, ignoring if it already exists"

		key = self.__resolve_keypath(key)

		if type(value) == str:
			self.client.set_string(key, value)

		elif type(value) == bool:
			self.client.set_bool(key, value)

		elif type(value) == int:
			self.client.set_int(key, value)

	def clear_mount_history(self):
		"Clears the mount history stored in gconf"
		self.client.set_list(self.__resolve_keypath(MOUNT_HISTORY), gconf.VALUE_STRING, [] )
		self.client.set_list(self.__resolve_keypath(DEV_HISTORY), gconf.VALUE_STRING, [] )
		self.client.set_list(self.__resolve_keypath(OPTIONS_HISTORY),gconf.VALUE_STRING, [])

	def get_history(self):
		try:
			v_nodes = self.get(DEV_HISTORY)
			m_nodes = self.get(MOUNT_HISTORY)
			o_nodes = self.get(OPTIONS_HISTORY)
			if (len(v_nodes) != len(m_nodes)) or len(v_nodes) != len(o_nodes) or len(m_nodes) != len(o_nodes):
				raise ConfigError
				print "History error length of lists does not match"
			history= []
			for x in range(len(v_nodes)):
				history+= [[v_nodes[x], m_nodes[x], self.buildOptions(o_nodes[x])]]
			return history
		except:
			return None
		
	def add_volume_to_gconf(self, volume):
		udi= volume.udi
		mp= volume.mount_point
		options= volume.options
		
		if udi == None or mp == None or options == None:
			return False
		
		v_nodes = self.get(DEV_HISTORY)
		m_nodes = self.get(MOUNT_HISTORY)
		o_nodes = self.get(OPTIONS_HISTORY)

		options_string= self.buildOptionsString(options)
		
		for i in range(len(v_nodes)):
			if v_nodes[i] == udi:
				m_nodes[i] = mp
				o_nodes[i] = options_string
				self.client.set_list(self.__resolve_keypath(MOUNT_HISTORY), gconf.VALUE_STRING, m_nodes)
				self.client.set_list(self.__resolve_keypath(OPTIONS_HISTORY),gconf.VALUE_STRING, o_nodes)
				return
			
		self.addToList(MOUNT_HISTORY, mp)
		self.addToList(DEV_HISTORY, udi)
		self.addToList(OPTIONS_HISTORY, options_string)
		return True
	
	def remove_volume_from_gconf(self, volume):
		udi= volume.udi
		v_nodes = self.get(DEV_HISTORY)
		m_nodes = self.get(MOUNT_HISTORY)
		o_nodes = self.get(OPTIONS_HISTORY)
		
		new_v = []
		new_m = []
		new_o = []
		for i in range(len(v_nodes)):
			if v_nodes[i] != udi:
				new_v.append(v_nodes[i])
				new_m.append(m_nodes[i])
				new_o.append(o_nodes[i])
				
		self.client.set_list(self.__resolve_keypath(DEV_HISTORY), gconf.VALUE_STRING, new_v)
		self.client.set_list(self.__resolve_keypath(MOUNT_HISTORY), gconf.VALUE_STRING, new_m)
		self.client.set_list(self.__resolve_keypath(OPTIONS_HISTORY),gconf.VALUE_STRING, new_o)
	
	def buildOptionsString(self, options):
		s= ""
		if options['user_mount'] == True:
			s+= "u_"
		else:
			s+= "-_"
		if options['read_only'] == True:
			s+= "r_"
		else:
			s+= "-_"
		if options['protect_hidden'] == True:
			s+= "p_"
		else:
			s+= "-_"
		if options['update_time'] == True:
			s+= "t_"
		else:
			s+= "-_"
		if options['filesystem'] == "" and options['filesystem'] not in MOUNT_FILESYSTEMS:
			s+= "auto_"
		else:
			s+= options['filesystem']+"_"
		if options['mount_options'] == "":
			s+= "-"
		else:
			s+= options['mount_options'] 
		return s
	
	def buildOptions(self, options_string):
		r_options= {}
		options= options_string.split("_")
		if options[0] == "u":
			r_options['user_mount'] = True
		else:
			r_options['user_mount'] = False
		if options[1] == "r":
			r_options['read_only'] = True
		else:
			r_options['read_only'] = False
		if options[2] == "p":
			r_options['protect_hidden'] = True
		else:
			r_options['protect_hidden'] = False
		if options[3] == "t":
			r_options['update_time'] = True
		else:
			r_options['update_time'] = False
		r_options['filesystem'] = options[4]
		if options[5] == "-":
			r_options['mount_options'] = ""
		else:
			r_options['mount_options'] = options[5]
		return r_options
	
	def iWantHistoryCaching(self):
		return self.get(CACHE_HISTORY)

def install_schema(file):
	"Installs a gconf schema"
	
	if os.access(file, os.F_OK) == False:
		return False
	
	output, status = utilRevelation.execute(FILE_GCONFTOOL + " --install-schema-file=" + file)
	return status == 0
