""" VOLUME CREATING DIALOG"""
import sys, gtk, os, popup, util, texpect, uiutil, truecryptResponse, config, utilRevelation
import volume
class CreateVolume:
    "TRUECRYPT OPTIONS"
    disable_progress = False
    display_keys = False
    quick_format = False
    
    #GUI OPTIONS
    truecrypt = "truecrypt "
    
    encryption_methods = ["AES", "Blowfish", "CAST5", "Serpent",
                          "Triple DES", "Twofish",\
                          "AES-Twofish","AES-Twofish-Serpent",\
                          "Serpent-AES", "Serpent-Twofish-AES",\
                          "Twofish-Serpent"]
    
    hash_methods = ["RIPEMD-160", "SHA-1", "Whirlpool"]
    
    filesystems = [_("FAT"), _("None"), "ext3"]

    types = [_("normal"), _("hidden")]
    
    units = [_("Megabyte"), _("Gigabyte"), _("Kilobyte"), _("Bytes")]
    
    cluster = 0
    
    path = os.path.abspath(sys.path[0])
    
    def __init__(self, trayApp):
        self.trayApp= trayApp
        self.gladefile = config.DIR_UI+"/dialogCreate.glade" 
        self.wtree = gtk.glade.XML(self.gladefile)
        self.window = self.wtree.get_widget("createWindow")
        

        self.wfilename = self.wtree.get_widget("filename")
        self.wsize = self.wtree.get_widget("size")
        self.wsize_unit = self.wtree.get_widget("size_unit")
        self.wcluster = self.wtree.get_widget("cluster")
        self.wencryption = self.wtree.get_widget("encryption")
        self.whash = self.wtree.get_widget("hash")
        self.wfilesystem = self.wtree.get_widget("filesystem")
        #self.wdisplay_keys = self.wtree.get_widget("display_keys")
        self.wquick_format = self.wtree.get_widget("quick_format")
        self.wtype = self.wtree.get_widget("type")
        self.wdir_chooser = self.wtree.get_widget("dir_chooser")
        
        self.wpassword = self.wtree.get_widget("password")
        self.wpass_label = self.wtree.get_widget("label10")
        self.w_show_password= self.wtree.get_widget("checkbutton1")
        self.eventbox= self.wtree.get_widget("eventbox1")
        
        self.wfiles = self.wtree.get_widget("keyfiles")
        self.create_progress= self.wtree.get_widget("progressbar1")
        
        self.liststore = gtk.ListStore(str)
        self.treeview = self.wfiles
        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.treeview.set_model(self.liststore)
        self.cell = gtk.CellRendererText()

        # create the TreeViewColumns to display the data
        self.tvcolumn = gtk.TreeViewColumn('File', self.cell, text=0)

        # add columns to treeview
        self.treeview.append_column(self.tvcolumn)
        self.treeview.set_search_column(0)

        # Allow sorting on the column
        self.tvcolumn.set_sort_column_id(0)

        # Allow enable drag and drop of rows including row move
        self.treeview.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
                                                uiutil.TARGETS,
                                                gtk.gdk.ACTION_DEFAULT|
                                                gtk.gdk.ACTION_MOVE)
        self.treeview.enable_model_drag_dest(uiutil.TARGETS,
                                             gtk.gdk.ACTION_DEFAULT)
        
        dic = {
           "on_checkbutton1_toggled" : self.show_pass_toggle,
           "on_password_changed" : self.check_password,
           "on_create_clicked" : self.handle_create,
           "on_disable_progress_toggled" : self.handle_button_toggeled,
           "on_display_keys_toggled" : self.handle_button_toggeled,
           "on_quick_format_toggled" : self.handle_button_toggeled,
           "on_close_clicked" : self.handle_close_clicked,
           "on_createWindow_destroy": self.handle_close_clicked,
           "on_keyfiles_drag_data_received" : uiutil.drag_data_received_data,
           "on_keyfiles_drag_data_get" : uiutil.drag_data_get_data,
           "on_clear_selected_clicked" : self.clear_selected,
           "on_clearall_clicked" : self.clear_all,
           "on_add_keyfile_clicked": self.keyfiles_chooser,
           "on_generate_keyfile_clicked": self.handle_generate_keyfile,
           "on_add_path_clicked" : self.keyfiles_chooser,
           "on_dir_chooser_current_folder_changed": self.dir_changed
           }
        self.wtree.signal_autoconnect(dic)
        self.set_init_options()
    
    def dir_changed(self, w):
        uri = self.wdir_chooser.get_current_folder_uri()
        uri = util.get_filepath_from_uri(uri) + "/"
        self.wfilename.set_text(uri)
    
    def keyfiles_chooser(self, w):
        choosePath= False
        if w.get_name() == "add_path":
            choosePath= True
        keyfile, response= uiutil.KeyfilesChooser(choosePath).run()
        if response != gtk.RESPONSE_CANCEL:
            self.treeview.get_model().append([keyfile])
    
    def handle_generate_keyfile(self, w):
        ""
    
    def set_init_options(self):
        
        "Initalize the widget with default values"
        for index in self.encryption_methods:
            self.wencryption.append_text( index )
        
        self.wencryption.set_active(0)
        
        for index in self.hash_methods:
            self.whash.append_text( index )
        
        self.whash.set_active(0)
        
        for index in self.filesystems:
            self.wfilesystem.append_text( index )
            
        self.wfilesystem.set_active(0)
        
        for index in self.units:
            self.wsize_unit.append_text( index )
        
        self.wsize_unit.set_active(0)
        
        for index in self.types:
            self.wtype.append_text( index )
        
        self.wcluster.set_value(self.cluster)
        #Set first value as active default
        self.wtype.set_active(0)

        #self.wdisplay_keys.set_active(util.if_else_bool(self.display_keys))
        self.wquick_format.set_active(util.if_else_bool(self.quick_format))
        
        #Set the icon
        self.window.set_icon_from_file(uiutil.ICON_PATH)
        
        #Set default path to user home directory
        self.wdir_chooser.set_current_folder_uri(os.path.expanduser("~"))
    
    def show_pass_toggle(self, w):
        self.display_password = not(self.w_show_password)
        self.wpassword.set_visibility(not(self.wpassword.get_visibility()))

        
    def check_password(self, w):
        #if len(password) == 0:
        #    self.remove_icon()
        password= w.get_text()
        try:
            utilRevelation.check_password(password)

        except ValueError, reason:
            self.wpass_label.set_text(_('The password %s') % str(reason))
            self.eventbox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.Color(65535, 0 , 0 ,0))
        else:
            self.wpass_label.set_text(_('The password seems good'))
            self.eventbox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.Color(0, 65535 , 0 ,0))
            
    
    def clear_all(self, _bb):
        "Clears the list of keyfiles"
        gtk.ListStore.clear(self.liststore)
    
    
    def clear_selected(self, button):
        "Clears the selected keyfile"
        selection = self.treeview.get_selection()
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for _iter in iters:
            model.remove(_iter)
            
    def handle_button_toggeled(self, widget):
        "Toggle internal value state for given widget"
        
        wname = widget.get_name()

        if wname == "display_keys":
            self.display_keys = not(self.display_keys)
        elif wname == "quick_format":
            self.quick_format = not(self.quick_format)
        else:
            ""
              
    def handle_close_clicked(self, widget):
        "Destorys the window when close is clicked"
        self.window.destroy()
    
    def update_progress(self, percent, string):
        self.create_progress.set_fraction(percent)
        self.create_progress.set_text(string)
        while gtk.events_pending():
           gtk.main_iteration()
        
    def handle_create(self, widget):
        "Creates a truecrypt volume \
        Complains if the necessary member values are not set \
        Error Handler, uses texpect"
        
        filesystem = self.wfilesystem.get_active_text()
        #Form error handling
        if self.wfilename.get_text() == "":
            popup.Popup(_("Give a filename!"))
        
        elif self.wsize.get_text() == "" or not self.wsize.get_text().isdigit():
            popup.Popup(_("Invalid size given"))
            
        elif filesystem != _("FAT") and filesystem != _("vfat") and self.quick_format:
            popup.Popup(_("Quick format can only be done on FAT Volumes"))
            
        elif self.wpassword.get_text() == "" \
                and self.liststore.get_iter_root() == None:
            
            popup.Popup(_("No password or keyfile(s) given"))
        else:
            _file = self.wfilename.get_text()
            if _file.startswith("/dev"):
                result= popup.Popup(_("Do you really want to create a truecrypt volume on a device?"), popup.Popup.CANCEL)
                if result == False:
                    return
                
            #Set various options via commandline arguments
            self.cluster_size= None
            if self.wcluster.get_value_as_int() != 0:
                self.cluster_size= `self.wcluster.get_value_as_int()`

            if self.wsize_unit.get_active_text() == "Megabyte":
                self.size_unit= "M"
            elif self.wsize_unit.get_active_text() == "Gigabyte":
                self.size_unit= "G"
            elif self.wsize_unit.get_active_text() == "Kilobyte":
                self.size_unit= "K"
            else:
                self.size_unit= None
            
            progressbar = uiutil.ProgressBar()
            pw = self.wpassword.get_text()
            exitstatus, vol= volume.Volume.create(     
                       "/"+_file.lstrip('file:/'),
                       pw,
                       self.wsize.get_text(),
                       self.size_unit,        
                       self.wencryption.get_active_text(),
                       self.whash.get_active_text(),
                       self.wtype.get_active_text(),
                       filesystem,
                       uiutil.get_list_of_treemodel(self.liststore),
                       self.cluster_size,
                       self.display_keys,
                       self.quick_format,
                       progressbar,
                       uiutil.overwriteFile,
                       uiutil.enterchars,
                       self.update_progress,
                        )
            if vol.isFileVolume():
                vol.fixUserRights()
            
            
            if filesystem != _("FAT") and filesystem != _("None") and not exitstatus== truecryptResponse.DO_NOT_OVERWRITE:
                #TODO: really true?
                status = vol.format(filesystem, True, uiutil.get_list_of_treemodel(self.liststore), pw)
                if status== True:
                    popup.Popup(_("Volume created"))
                    
            if exitstatus == truecryptResponse.CANCELED:
                return
            elif exitstatus == truecryptResponse.SIZE_NOT_FAT:
                popup.Popup(_("Unable to create fat filesystem with that size"))
            elif exitstatus == truecryptResponse.OUTER_VOLUME_TO_SMALL:
                popup.Popup(_("Outer volume to small for specified hidden volume size"))
            elif exitstatus == truecryptResponse.VOLUME_CREATED:
                popup.Popup(_("Volume created"))
            elif exitstatus == truecryptResponse.TRY_AGAIN:
                popup.Popup(_("Wrong sudo password"))
            elif exitstatus == truecryptResponse.NOT_A_DIRECTORY:
                popup.Popup(_("Not a directory"))
            elif exitstatus == truecryptResponse.VOLUME_EXISTS:
                popup.Popup(_("Volume exists"))
        
if __name__ == "__main__":
    HWG = CreateVolume()
    gtk.main()
