"""GUI REPRESNTATION OF LIST OF MOUNTED VOLUMES"""
import popup, sys, gtk, os, util, pexpect, uiutil, config, gobject, truecryptResponse
import dialogVolumePrefs
#from threading import Thread

class TruecryptList:
    path = os.path.abspath(sys.path[0])
    visible= True
    
    def __init__(self, trayApp= None, mounted=None):
        self.trayApp= trayApp
        
        self.truecrypt= config.Config().get('command')
        self.gladefile=config.DIR_UI+"/dialogList.glade" 
        self.wtree=gtk.glade.XML(self.gladefile)
        self.window = self.wtree.get_widget("mountedWindow")
        self.window.set_icon_from_file(uiutil.ICON_PATH)
        
        self.wlist = self.wtree.get_widget("mountedVolumes")
        
        self.liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.treeview= self.wlist
        self.treeview.get_selection().set_mode(gtk.SELECTION_SINGLE)
        self.treeview.set_model(self.liststore)
        self.cell = gtk.CellRendererText()

        # create the TreeViewColumns to display the data
        self.tvcolumn = gtk.TreeViewColumn('Volume', self.cell, text=0)
        self.tvcolumn1 = gtk.TreeViewColumn('Mountpoint', self.cell, text=1)
        # add columns to treeview
        self.treeview.append_column(self.tvcolumn)
        self.treeview.append_column(self.tvcolumn1)
        
        self.eventbox= self.wtree.get_widget("eventbox1")
        self.eventlabel= self.wtree.get_widget("label1")
        self.mount_mapped= self.wtree.get_widget("mount_mapped_volume")
        self.information= self.wtree.get_widget("properties")
        self.unmount_volume= self.wtree.get_widget("unmount_selected")
        self.unmount_all= self.wtree.get_widget("unmount_all")
        self.refreshVolumeList(None,True, mounted)
        self.volume_menu= self.wtree.get_widget("volumeMenu")
        
        self.menu_properties= self.wtree.get_widget("menu_properties")
        self.menu_unmount=self.wtree.get_widget("menu_unmount")
        self.menu_mount_mapped=self.wtree.get_widget("menu_mountmapped")
        
        dic = {
            "on_unmount_selected_activate" : self.handle_unmount_selected,
            "on_unmount_all_activate" : self.handle_dismount_all,
            "on_properties_activate": self.handle_properties,
            "on_preferences_activate": self.handle_preferences,
            "on_refresh_activate" : self.refreshVolumeList,
            "on_close_activate" : self.handle_close_clicked,
            "on_mountedWindow_destroy": self.handle_close_clicked,
            "on_mount_activate" : self.handle_mount_volume,
            "on_create_activate": self.handle_create_volume,
            "on_clear_activate" : self.handle_clear,
            "on_global_preferences_activate" : self.handle_glob_preferences,
            "on_mount_mapped_volume_activate": self.mount_mapped_volume,
            "on_volumes_row_activated": self.row_activated,
            "on_volumes_cursor_changed": self.row_activated,
            "on_info1_activate": self.trayApp.about,
            "on_volumes_button_press_event": self.on_volume_menu,
            "on_menu_mountmapped_activate":self.mount_mapped_volume,
            "on_menu_properties_activate":self.handle_properties,
            "on_menu_unmount_activate":self.handle_unmount_selected
           }
           
        self.wtree.signal_autoconnect(dic)
        self.window.show_all()
        
    def on_volume_menu(self, view, event):

        if event.button == 3:
            path, col, cellx, celly = view.get_path_at_pos(int(event.x),
                                                           int(event.y))

            view.grab_focus()
            selection = view.get_selection()
            
            if not selection.path_is_selected(path):
                view.set_cursor( path, col, 0)
            model, iter= selection.get_selected()
            firstColumn= model.get_value(iter, 0)
            vol= self.trayApp.volumes_obj.by_volume_path(firstColumn)
            if vol== False:
                return
            
            isMounted=vol.isMounted()
            isMapped=vol.isMapped()
            
            self.menu_properties.set_sensitive(isMounted)
            self.menu_unmount.set_sensitive(isMounted)
            self.menu_mount_mapped.set_sensitive(isMapped and not isMounted)
            self.volume_menu.popup(None,None,None,event.button,event.time,None)
            return True
        return False
    
    def mount_mapped_volume(self, w):
        selection = self.treeview.get_selection()
        if selection.count_selected_rows() == 0:
            self.error_feedback(_("No volumes mapped or selected")) 
            return False
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            vol= model.get_value(iter, 0)
            vol= self.trayApp.volumes_obj.by_volume_path(vol)
            if vol.isMounted():
                self.error_feedback(_("Volume already mounted")) 
                return False
            #TODO: grey out in select
            result, mount_point= uiutil.Mount_Point_Dialog(vol.mount_point).run()
            if result == gtk.RESPONSE_CANCEL:
                return
            
            status= vol.mount_mapped(mount_point)
            
            if status == pexpect.EOF:    
                model.remove(iter)
                self.ok_feedback(_("Mounting of %s successful") % mountpoint)
            elif status == truecryptResponse.TRY_AGAIN:
                self.error_feedback(_("Wrong sudo password")) 
            elif status == truecryptResponse.CANCELED:
                return
            
        self.refreshVolumeList(None, False)
    
    def handle_close_clicked(self, w):
        self.isvisible= False
        self.window.hide()
    
    def handle_glob_preferences(self, w):
        self.trayApp.propertiesDialogProxy()
        
    def handle_mount_volume(self, w):
        self.trayApp.mountDialogProxy()
        
    def handle_create_volume(self, w):
        self.trayApp.createDialogProxy()   

    def handle_clear(self, w):
        self.trayApp.clearHistory()
    
    def ok_feedback(self, string):
        self.eventbox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.Color(0, 65535 , 0 ,0))
        self.eventlabel.set_text(string)         
    
    def error_feedback(self, string):
        self.eventbox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.Color(65535, 0 , 0 ,0))
        self.eventlabel.set_text(string) 
    
    def handle_dismount_all(self, w):
        result= self.trayApp.volumes_obj.dismount_all()
        
        self.refreshVolumeList()
        if result == truecryptResponse.DISMOUNTING_SUCCESSFULL:
            self.ok_feedback(_("Dismounting successful!"))
        elif result == truecryptResponse.TRY_AGAIN:
            self.error_feedback(_("Wrong sudo password"))
        elif result == truecryptResponse.CANCELED:
            return
        elif result == truecryptResponse.NO_VOLUMES_MAPPED:
            self.error_feedback(_("No volumes mapped"))
        elif result == truecryptResponse.DEVICE_BUSY:
            self.error_feedback(_("Some volume could not be dismounted, device busy"))
    
    def handle_preferences(self, w):
        dialogVolumePrefs.VolumePreferences(self.trayApp)
    
    def handle_unmount_selected(self, w):
        selection = self.treeview.get_selection()
        if selection.count_selected_rows() == 0:
            self.error_feedback(_("No volumes mapped or selected")) 
            return False
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            volume= model.get_value(iter, 0)
            mountpoint= model.get_value(iter, 1)
            vol = self.trayApp.volumes_obj.by_volume_path(volume)
            if vol == False:
                return
            status= vol.unmount()
            
            if status == pexpect.EOF:    
                model.remove(iter)
                self.ok_feedback(_("Dismounting of %s successful") % mountpoint)
            elif status == truecryptResponse.TRY_AGAIN:
                self.error_feedback(_("Wrong sudo password"))
            elif status == truecryptResponse.CANCELED:
                return
            elif status == truecryptResponse.DEVICE_BUSY:
                self.error_feedback(_("The volume %s could not be dismounted, device busy") %mountpoint)
            
        self.refreshVolumeList(None, False)

    def row_activated(self, w, n=None, p=None):
        selection = self.treeview.get_selection()

        if selection.count_selected_rows() == 0:
            self.mount_mapped.set_sensitive(False)
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            volume= model.get_value(iter, 0)
            if volume == _("No volumes mapped"):
                mounted= False
                mapped= False            
                return
            mountpoint= model.get_value(iter, 1)
            try:
                vol= self.trayApp.volumes_obj.by_volume_path(volume)
            except:
                mounted= False
                mapped= False
            try:
                mounted= vol.isMounted()
                mapped= vol.isMapped()
            except:
                mounted= False
                mapped= False

            self.mount_mapped.set_sensitive(mounted)
            self.information.set_sensitive(mapped)
            self.unmount_volume.set_sensitive(mapped)
    
    def refreshVolumeList(self, w=None, feedback=True, mounted= None):
        self.liststore.clear()
        if mounted != None:
            xdata= mounted
        else:
            xdata= self.trayApp.mountedVolumes()
        
        if xdata== truecryptResponse.NO_VOLUMES_MAPPED or xdata == [] or xdata == None:
            xdata= [[_("No volumes mapped"), ""]]
            self.unmount_all.set_sensitive(False)
            if feedback: self.ok_feedback(_("Refreshed: No volumes mapped"))
        else:
            if feedback: self.ok_feedback(_("Refreshed"))
            self.unmount_all.set_sensitive(True)
        [self.liststore.append(xdata[i]) for i in range(len(xdata))]
        

    def handle_properties(self, w):
        allvolumes = uiutil.get_list_of_treemodel(self.liststore)
        
        selection = self.treeview.get_selection()
        model, selected = selection.get_selected_rows()
        
        if allvolumes == []:
            self.error_feedback(_("No volumes mapped"))
            return False
        
        elif selection.count_selected_rows() == 0:
            self.error_feedback(_("No volumes selected"))
            return False  
        
        
        
        iters = [model.get_iter(path) for path in selected]
        volume = None
        for iter in iters:
            volume= model.get_value(iter, 0)       
        
        result, passw= uiutil.PasswordDialog(_("Password for volume ")+volume, "Please enter password").run()
        result, listkeyfiles= uiutil.EnterKeyfiles().run()
        keyfiles= uiutil.get_list_of_treemodel( listkeyfiles)
        
        if keyfiles == [] and passw== "":
            return
        exitCode, props= self.trayApp.volumes_obj.by_volume_path(volume).properties(passw, keyfiles)

        if exitCode == truecryptResponse.CANNOT_OPEN_VOLUME:
            self.error_feedback(_("Can not open volume"))
        elif exitCode == truecryptResponse.TRY_AGAIN:
            self.error_feedback(_("Wrong sudo password"))
        elif exitCode == truecryptResponse.CANCELED:
            return
        elif exitCode== truecryptResponse.PROPERTIES:
            popup.Popup(props, title=_("Properties of ")+volume)
        elif exitCode== truecryptResponse.INCORRECT_VOLUME:
            popup.Popup(_("Incorrect password or not a truecrypt volume"),title=_("Incorrect password or not a truecrypt volume"))
        else:
            popup.Popup(_("Incorrect password or not a truecrypt volume"),title=_("Unknown exit code"))
        
    def show(self):
        self.visible= True
        self.window.show()
        
    def hide(self):
        self.visible= False
        self.window.hide()
        
    def isvisible(self):
        return self.visible

        
if __name__ == "__main__":
    hwg = TruecryptList()
    gtk.main()