import __builtin__, gettext
__builtin__._ = gettext.gettext

import gtk, os, config, sys, uiutil, gobject, truecryptResponse
from gtk import glade

class VolumePreferences:
    path = os.path.abspath(sys.path[0])
    visible= True
    def __init__(self, trayApp= None):
        self.trayApp= trayApp

        self.gladefile=config.DIR_UI+"/dialogVolumePreferences.glade" 
        self.wtree=gtk.glade.XML(self.gladefile)
        self.window = self.wtree.get_widget("volume_preferences")
        self.window.set_icon_from_file(uiutil.ICON_PATH)
        
        self.wlist = self.wtree.get_widget("volumes")
        self.liststore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gtk.gdk.Pixbuf)
        self.treeview= self.wlist
        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.treeview.set_model(self.liststore)
        self.cell = gtk.CellRendererText()
        self.img_cell = gtk.CellRendererPixbuf()
        
        # create the TreeViewColumns to display the data
        self.tvcolumn = gtk.TreeViewColumn(_('Volume UID'), self.cell, text=0)
        self.tvcolumn1 = gtk.TreeViewColumn(_('Mountpoint'), self.cell, text=1)
        self.mount_column= gtk.TreeViewColumn(_('Mounted'), self.img_cell, pixbuf=2)

        self.mounted_image = self.window.render_icon(gtk.STOCK_YES, gtk.ICON_SIZE_MENU)
        self.not_mounted_image = self.window.render_icon(gtk.STOCK_NO, gtk.ICON_SIZE_MENU)
        self.mapped_image = self.window.render_icon(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_MENU)
        
        # add columns to treeview
        self.treeview.append_column(self.tvcolumn)
        self.treeview.append_column(self.tvcolumn1)
        self.treeview.append_column(self.mount_column)
        
        self.filesystem = self.wtree.get_widget('filesystem')
        self.user_mount = self.wtree.get_widget('user_mount')
        self.update_time = self.wtree.get_widget('update_time')
        self.protect_hidden = self.wtree.get_widget('protect_hidden')
        self.read_only = self.wtree.get_widget('read_only')
        self.mount_options = self.wtree.get_widget('mount_options')
        
        self.removeVolumeButton= self.wtree.get_widget('removeVolume')

        
        for fs in config.MOUNT_FILESYSTEMS:
            self.filesystem.append_text(fs)
        
        dic = {
            "on_volume_preferences_close_clicked" : self.handle_close_clicked,
            "on_volume_preferences_destroy": self.handle_close_clicked,
            "on_cancel_clicked" : self.handle_close_clicked,
            "on_apply_clicked" : self.handle_apply_clicked,
            "on_ok_clicked" : self.handle_ok_clicked,
            #"on_volumes_row_activated" : self.handle_row_activated,
            "on_volumes_cursor_changed" : self.handle_row_activated,
            "on_removeVolume_clicked" : self.handle_remove_volume
           }
        
        self.wtree.signal_autoconnect(dic)
        self.volumes_list = self.trayApp.volumes_obj.volume_list
        
        self.refreshList()
        
        self.window.show_all()
        
    def handle_close_clicked(self, w):
        self.window.destroy()
    
    def refreshList(self):
        self.liststore.clear()
        all_volumes= []
        self.volumes_list, status= self.trayApp.volumes_obj.refresh()
        if status == truecryptResponse.TRY_AGAIN:
            popup.Popup(_("Wrong sudo password"))
        
        self.volumes_list= self.volumes_list.volume_list
        if self.volumes_list == []:
            sensitive= False
        else:
            sensitive= True
        
        self.filesystem.set_sensitive(sensitive)
        self.user_mount.set_sensitive(sensitive)
        self.update_time.set_sensitive(sensitive)
        self.protect_hidden.set_sensitive(sensitive)
        self.read_only.set_sensitive(sensitive)
        self.mount_options.set_sensitive(sensitive)
        self.removeVolumeButton.set_sensitive(sensitive)

            
        for volume in self.volumes_list:
            if volume.isMounted():
                image= self.mounted_image
            elif volume.isMapped():
                image= self.mapped_image
            else:
                image= self.not_mounted_image
            all_volumes+= [[volume.udi, volume.mount_point, image]]
        [self.liststore.append(all_volumes[i]) for i in range(len(all_volumes))]
        
        
    def handle_apply_clicked(self, w):
        options= {'user_mount'    : self.user_mount.get_active(),
                  'read_only'     : self.read_only.get_active(),
                  'protect_hidden': self.protect_hidden.get_active(),
                  'update_time' : self.update_time.get_active(),
                  'filesystem'    : self.filesystem.get_active_text(),
                  'mount_options' : self.mount_options.get_text()
                  }
        selection = self.treeview.get_selection()
        if selection.count_selected_rows() == 0:
            return False
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            udi= model.get_value(iter, 0)
            mountpoint= model.get_value(iter, 1)
            volume= self.trayApp.volumes_obj.by_udi(udi)
            if volume != False:
                status= volume.set_options(options)
        
        
    def handle_ok_clicked(self, w):
        self.handle_apply_clicked(None)
        self.handle_close_clicked(None)
        
    def handle_row_activated(self, w, b=None, s=None):
        "load preferences"
        selection = self.treeview.get_selection()
        if selection.count_selected_rows() == 0:
            return False
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            udi= model.get_value(iter, 0)
            mountpoint= model.get_value(iter, 1)
            volume= self.trayApp.volumes_obj.by_udi(udi)
            if volume.isMapped() or volume.isMounted():
                self.removeVolumeButton.set_sensitive(False)
            else:
                self.removeVolumeButton.set_sensitive(True)
            if volume != False:
                self.show_options(volume.options)
                
    def show_options(self, options):
        model= self.filesystem.get_model()
        iter = model.get_iter_first()
        i= 0
        while iter != None:
            value= model.get_value(iter, 0)
            
            if value == options['filesystem']:
                self.filesystem.set_active(i)
            i= i+1
            iter= model.iter_next(iter)
        
        self.user_mount.set_active(options['user_mount'])
        self.update_time.set_active(options['update_time'])
        self.protect_hidden.set_active(options['protect_hidden'])
        self.read_only.set_active(options['read_only'])
        self.mount_options.set_text(options['mount_options'])
        
        
        
    def handle_remove_volume(self, w):
        selection = self.treeview.get_selection()
        if selection.count_selected_rows() == 0:
            return False
        model, selected = selection.get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        for iter in iters:
            udi= model.get_value(iter, 0)
            mountpoint= model.get_value(iter, 1)
            volume= self.trayApp.volumes_obj.remove_volume_from_history(udi)
        self.refreshList()

            
if __name__ == "__main__":
    hwg = VolumePreferences()
    gtk.main()