""" TEXPECT BACKEND FOR THE GUI"""
import sys, commands, truecryptResponse, os, config

import pexpect
import types
import zero_out

class Texpect:
    "expecting truecrypt input"
################################################################################
# MEMBERS - USE WISELY!
################################################################################
    conf = config.Config()
    command = conf.get(config.COMMAND)
    status = ""
    properties = ""
    count= 0
    
    def __init__(self, debug=True):
        if debug==True: 
            self.log=sys.stdout
            self.debug=True
        else: self.log=None
        ""
        
################################################################################
# EXECUTE A COMMAND AS SUPERUSER
# --
# @return:
#            truecryptResponse.TRY_AGAIN
#            truecryptResponse.CANCELED
#            False if an error occured
#            True if everything went alright
#            truecryptResponse.REVERSED_PATCH_DETECTED is for special patch command
################################################################################        
    def doSudoCommand(self, command, sudo_callback, message):
        self.status = ""
        command = "sudo "+command
        try:
            (command_output, exitstatus) = pexpect.run (command, 
                 events={
                         truecryptResponse.REVERSED_PATCH_DETECTED : lambda *args: self.patch_detected(*args),
                         truecryptResponse.ENTER_SUDO_PASSWORD     : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                         truecryptResponse.SUDO_PASSWORD           : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                         truecryptResponse.ENTER_PASSWORD_43A      : lambda *args: self.enter_sudo_password2(sudo_callback,message, *args),
                         truecryptResponse.TRY_AGAIN               : lambda *args: self.try_again(*args),
                         pexpect.EOF                               : lambda *args: self.eof( *args)
                         },
                 withexitstatus= True,
                 logfile=self.log
                 )
        except:
            exitstatus = 1
            pass
        
        if self.status == truecryptResponse.TRY_AGAIN:
            return self.status
        elif self.status == truecryptResponse.CANCELED:
            return self.status
        elif exitstatus != 0:
            return False
        return True
    
    def try_again(self, *args):
        self.status= truecryptResponse.TRY_AGAIN
        return self.close(*args)
    
    def enter_sudo_password2(self, sudo_callback, message, *args):
        if sudo_callback != None:
            result, passw= sudo_callback(message)
            if result == False:
                self.status= truecryptResponse.CANCELED
                return self.close(*args)
            return passw+"\n"
        else:
            return self.close(*args)        
   
    def patch_detected(*args):
        return "n\n"
    
################################################################################
# DISMOUNT
#
################################################################################
    def dismount(self, volume, sudo_callback, message):
        "execute to dismount"
        self.status = ""
        command= self.command+" -d "
        if volume != None:
            command = command+volume
        command= "sudo "+command
        (command_output, exitstatus) = pexpect.run (command, 
             events={
                     truecryptResponse.NO_VOLUMES_MAPPED       : lambda *args: self.no_volumes_mapped_on_listing(*args),
                     pexpect.EOF                               : lambda *args: self.no_volumes_mapped_on_dismount( *args),
                     truecryptResponse.ADMIN_RIGHTS_REQUIRED   : lambda *args: self.admin_required(*args),
                     truecryptResponse.ENTER_SUDO_PASSWORD     : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.SUDO_PASSWORD           : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.ENTER_PASSWORD_43A      : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.TRY_AGAIN               : lambda *args: self.try_again(*args),                     
                     truecryptResponse.DEVICE_BUSY             : lambda *args: self.device_busy(*args)
                     },
             withexitstatus= True,
             logfile=self.log
             )
        if self.status== truecryptResponse.ADMIN_RIGHTS_REQUIRED:
            return self.dismount(volume,sudo_callback,True)
        elif self.status== truecryptResponse.TRY_AGAIN:
            return self.status
        
        return self.status
    
    def device_busy(self, *args):
        self.status = truecryptResponse.DEVICE_BUSY
        return self.close(*args)
################################################################################
# CREATION
################################################################################
    def create(self,
                      volume,
                      password,
                      size,
                      size_unit,                      
                      encryption,
                      hash,                      
                      type,
                      filesystem,
                      keyfiles=None,
                      cluster_size=None,
                      display_keys=None,
                      quick_format= None,
                      sudo_callback= None,
                      message= None,
                      mouse_callback=None,
                      exists_callback=None,
                      key_callback=None,
                      progress_callback=None,
                      ):
        self.status= ""
        command= self.command+" "

        self.keyfiles= keyfiles
        
        if display_keys== True:
            command.append("--display-keys " )
        
        if quick_format== True:
            command.append("--quick ")
        if size_unit==None:
            command += "--size %s " % (size)
        else:
            command += "--size %s%s " % (size, size_unit)
        
        command += "--encryption %s " %encryption
        command += "--hash %s " %hash
        command += "--type %s " %type
        command += "--filesystem %s " % filesystem
        command += "-c %s" % volume
        command += ""
        command= "sudo "+command
        (command_output, exitstatus) = pexpect.run (command, 
             events={
                     truecryptResponse.ENTER_PASSWORD      : lambda *args: self.enter_password(password, *args),
                     truecryptResponse.REENTER_PASSWORD    : lambda *args: self.reenter_password(password, *args),
                     truecryptResponse.ENTER_KEYFILES      : lambda *args: self.enter_keyfile(*args),
                     truecryptResponse.MOUSE_CONNECTED     : lambda *args: self.mouse_connected(mouse_callback,*args),
                     truecryptResponse.PROGRESS_RANDOM     : lambda *args: self.mouse_movement(mouse_callback, *args),
                     truecryptResponse.VOLUME_EXISTS       : lambda *args: self.volume_exists(exists_callback, volume, *args),
                     truecryptResponse.CREATED_PATTERN     : lambda *args: self.volume_created(*args),
                     pexpect.EOF                           : lambda *args: self.eof(*args),
                     truecryptResponse.PLEASE_TYPE           : lambda *args: self.please_type(key_callback,*args),
                     truecryptResponse.CREATE_PROGRESS     : lambda *args: self.create_progress(progress_callback, size, size_unit, *args),
                     truecryptResponse.ENTER_SUDO_PASSWORD     : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.SUDO_PASSWORD           : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.ENTER_PASSWORD_43A      : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.TRY_AGAIN               : lambda *args: self.try_again(*args),
                     truecryptResponse.SIZE_NOT_FAT            : lambda *args: self.size_not_fat(*args),
                     truecryptResponse.OUTER_VOLUME_TO_SMALL   : lambda *args: self.outer_too_small(*args)
                     },
             withexitstatus= True,
             logfile=self.log
             )

        return self.status
    def size_not_fat(self, *args):
        self.status= truecryptResponse.SIZE_NOT_FAT
        return self.close()
    
    def please_type(self,key_callback, *args):
        result, chars= key_callback()
        if len(chars)<320:
            return self.close(*args)
        return self.enter(chars,*args)

    def outer_too_small(self, *args):
        self.status= truecryptResponse.OUTER_VOLUME_TO_SMALL
        return self.close(*args)
        
################################################################################
# VOLUMELIST PARSING
#
################################################################################
    
    def parsevolumelist_without_truecrypt(self):        
        fsi = mtpython.FileSystemInfo(mounted_only=1)
        mounts= []
    
        for i in fsi.get_fuzzy(spec='/dev/mapper/truecrypt'):
            mounts+= [[i.spec, i.file]]
        return mounts
    
    def parsevolumelist(self, sudo_callback=None, message=None):
        "returns -1 if no volumes are mapped"
        self.tmp= ""
        command= self.command+" -l"
        command= "sudo "+command
        (command_output, exitstatus) = pexpect.run (command, 
             events={
                     truecryptResponse.NO_VOLUMES_MAPPED   : lambda *args: self.no_volumes_mapped_on_listing(*args),
                     truecryptResponse.LISTPATTERN         : lambda *args: self.parse_volume(*args),
                     truecryptResponse.ENTER_SUDO_PASSWORD     : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.SUDO_PASSWORD           : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.ENTER_PASSWORD_43A      : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.TRY_AGAIN               : lambda *args: self.try_again(*args),
                     pexpect.EOF                           : lambda *args: self.eof( *args)
                     },
             withexitstatus= True,
             logfile=self.log
             )
        if self.status== truecryptResponse.TRY_AGAIN:
            return truecryptResponse.TRY_AGAIN
        elif self.status== truecryptResponse.NO_VOLUMES_MAPPED:
            return truecryptResponse.NO_VOLUMES_MAPPED
        
        mapped= []
        mapped= self.tmp.split("\r\n")
        for i in range(len(mapped)):
            if mapped[i] != "":
                mapped[i]= mapped[i].split(" ")
            else:
                mapped.remove(mapped[i])
        return mapped
    
    def admin_required(self, *args):
        self.status= truecryptResponse.ADMIN_RIGHTS_REQUIRED
        return self.close(*args)
        
    def properties(self, volume, password, keyfiles, sudo_callback, sudo=False):
        "output handler accepts string as properties for now"
        self.keyfiles= keyfiles
        self.status = ""
        self.properties= None
        command= self.command+" --properties"
        command= "sudo "+command
        (command_output, exitstatus) = pexpect.run (command, 
             events={
                     truecryptResponse.ENTER_VOLUME        : lambda *args: self.enter_volume(volume, *args),#NOTE: this is ok
                     truecryptResponse.ENTER_KEYFILES      : lambda *args: self.enter_keyfile(*args),
                     truecryptResponse.CANNOT_OPEN_VOLUME  : lambda *args: self.cannot_open_volume(*args),
                     truecryptResponse.ENTER_PASSWORD      : lambda *args: self.enter_password(password, *args),
                     truecryptResponse.PROPERTIES          : lambda *args: self.getproperties(*args),
                     truecryptResponse.INCORRECT_VOLUME    : lambda *args: self.incorrect_volume(*args),
                     truecryptResponse.ENTER_SUDO_PASSWORD     : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.SUDO_PASSWORD           : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.ENTER_PASSWORD_43A      : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.TRY_AGAIN               : lambda *args: self.try_again(*args)                  
                     },
             withexitstatus= True,
             logfile=self.log
             )
        
        self.count= 0
        if self.properties == None:
            return self.status, "No properties"
        return self.status, self.properties.lstrip().rstrip()
    
   
    
    def getproperties(self, *args):
        self.properties= args[0]['child'].after
        self.status= truecryptResponse.PROPERTIES
        return self.close(*args)




################################################################################
# MOUNTING
# 
################################################################################


    def mount(self,
              volume,
              mountdir,
              protected,
              keyfiles,
              password,
              filesystem= "auto",
              mount_options= None,
              read_only= False,
              user_mount= False,
              update_time=False,
              sudo_callback=False,
              message= None,
              hidden_password=None,
              hidden_keyfiles=None
               ):
        "mount interactive sequence implemented with events"
        self.status = ""
        self.keyfiles= keyfiles
        self.hidden_keyfiles= hidden_keyfiles
        command= self.command+" "
        
        if filesystem != "auto":
            command= command+"--filesystem "+filesystem+" "
            
        if read_only:
            command= command+"-r "
            
        if user_mount:
            #if sudo==False:
            command= command+"-u "
            #TODO: test if this works
            #else:
            #    mount_options="uid="+`os.getuid()`+",gid="+`os.getgid()`+" "
                
        if mount_options != "":
            command= command+"-M "+mount_options+" "
            
        if update_time:
            command= command+"--update-time "
        
        command= "sudo "+command
        command= command+"-i"
        if self.debug:
            print command
        (command_output, exitstatus) = pexpect.run (command, 
             events={
                     truecryptResponse.ENTER_VOLUME                : lambda *args: self.enter_volume(volume, *args),
                     truecryptResponse.ENTER_MOUNTPATH             : lambda *args: self.enter_mountpath(mountdir, *args),
                     truecryptResponse.ENTER_PROTECTION            : lambda *args: self.enter_protect(protected, *args),
                     truecryptResponse.ENTER_KEYFILES              : lambda *args: self.enter_keyfile(*args),
                     truecryptResponse.ENTER_HIDDEN_KEYFILES       : lambda *args: self.enter_hidden_keyfile(*args),                     
                     truecryptResponse.ENTER_PASSWORD              : lambda *args: self.enter_password(password, *args),
                     truecryptResponse.ENTER_HIDDEN_PASSWORD       : lambda *args: self.enter_hidden_password(hidden_password, *args),                     

                     truecryptResponse.ALREADY_MAPPED              : lambda *args: self.already_mapped(*args),
                     truecryptResponse.CANNOT_OPEN_VOLUME          : lambda *args: self.cannot_open_volume(*args),        #get exit status
                     truecryptResponse.INCORRECT_VOLUME            : lambda *args: self.incorrect_volume(*args),
                     truecryptResponse.ILLEGAL_SEEK                : lambda *args: self.illegal_seek(*args),        #exit
                     truecryptResponse.MOUNT_POINT_DOES_NOT_EXIST  : lambda *args: self.mountpoint_does_not_exist(*args), #ext
                     truecryptResponse.INCORRECT_VOLUME            : lambda *args: self.incorrect_volume(*args),
                     truecryptResponse.KEYFILE_DOES_NOT_EXIST      : lambda *args: self.keyfile_does_not_exists(*args),
                     pexpect.EOF                                   : lambda *args: self.eof(*args),
                     truecryptResponse.WRONG_FS_TYPE               : lambda *args: self.wrong_fs_type(*args),

                     truecryptResponse.ENTER_SUDO_PASSWORD         : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.SUDO_PASSWORD               : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.ENTER_PASSWORD_43A          : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                     truecryptResponse.TRY_AGAIN                   : lambda *args: self.try_again(*args)    
                     },
             withexitstatus= True,
             logfile= self.log
             )
        
        if exitstatus== 0:
            return truecryptResponse.VOLUME_MOUNTED
        elif self.status== truecryptResponse.TRY_AGAIN:
            return self.status        
        else:
            return self.status
        
    def mount_mapped(self, volume, mount_point, sudo_callback, message):
        mapped= volume.mapper_device
        try:
            command= "sudo mount "+mapped+" "+mount_point
            (command_output, exitstatus) = pexpect.run (command, 
                 events={
                         
                         pexpect.EOF                           : lambda *args: self.eof(*args),
                         truecryptResponse.ENTER_SUDO_PASSWORD         : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                         truecryptResponse.SUDO_PASSWORD               : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                         truecryptResponse.ENTER_PASSWORD_43A          : lambda *args: self.enter_sudo_password2(sudo_callback, message, *args),
                         truecryptResponse.TRY_AGAIN                   : lambda *args: self.try_again(*args)    
    
                         },
                 withexitstatus= True,
                 logfile= self.log
                 )
        except:
            ""
        if exitstatus== 0:
            return truecryptResponse.VOLUME_MOUNTED
        else:
            return self.status
################################################################################
# ACTIONS
#  
################################################################################

################################################################################
# ENTER FUNCTIONS
################################################################################
    def incorrect_volume(self, *args):
        self.status= truecryptResponse.INCORRECT_VOLUME
        return self.close(*args)
    
    def enter(self, string, *args):
        return string+"\n"
    
    def enter_volume(self, string, *args):
        self.status= truecryptResponse.ENTER_VOLUME
        if self.count== 3:
            return self.close(*args)
        self.count= self.count + 1
        return self.enter(string, *args)
    
    def enter_mountpath(self, string, *args):
        self.status= truecryptResponse.ENTER_MOUNTPATH
        return self.enter(string, *args)
        
    def enter_protect(self,protect, *args):
        self.status= truecryptResponse.ENTER_PROTECTION
        if protect:
            return "y\n"
        else:
            return "n\n"

    def enter_hidden_keyfile(self, *args):
        self.status= truecryptResponse.ENTER_HIDDEN_KEYFILES
        if len(self.hidden_keyfiles) != 0 and self.hidden_keyfiles!=None:
            file= self.hidden_keyfiles.pop()
            return file+"\n"
        else:
            return "\n" 
        
    def enter_hidden_password(self, password, *args):
        self.status= truecryptResponse.ENTER_HIDDEN_PASSWORD
        result = self.enter(password, *args)
        zero_out.zero_out(password)
        return result
        
    def enter_password(self, password, *args):
        self.status= truecryptResponse.ENTER_PASSWORD
        return self.enter(password, *args)

    
    def reenter_password(self, password, *args):
        self.status= truecryptResponse.ENTER_PASSWORD
        result= self.enter(password, *args)
        #zero_out.zero_out(password)
        #TODO:MAKE MOR INTELIGENT
        return result
    
    def enter_newline(self, *args):
        return "\n"       

    def enter_sudo_password(self, sudo_callback, *args):
        if sudo_callback != None:
            passw= sudo_callback()
            return passw+"\n"
        else:
            return self.close(*args)
    
    def enter_keyfile(self, *args):
        self.status= truecryptResponse.ENTER_KEYFILES
        if len(self.keyfiles) != 0 and self.keyfiles!=None:
            file= self.keyfiles.pop()
            return file+"\n"
        else:
            return "\n" 
    
    def volume_exists(self, callback, file, *args):
        self.status= truecryptResponse.VOLUME_EXISTS
        if callback(file):
            return "y\n"
        else:
            self.status= truecryptResponse.DO_NOT_OVERWRITE
            return "n\n"
            
################################################################################
# VARIOUS    
################################################################################
    def keyfile_does_not_exists(self, *args):
        self.status= truecryptResponse.KEYFILE_DOES_NOT_EXIST
        self.close(*args)
        
    def no_volumes_mapped_on_listing(self, *args):
        self.status= truecryptResponse.NO_VOLUMES_MAPPED
        return self.close(*args)
    
    def no_volumes_mapped_on_dismount(self, *args):
        self.status= truecryptResponse.DISMOUNTING_SUCCESSFULL
        return self.close(*args)
    
    def parse_volume(self, *args):
        self.tmp += args[0]['child'].after
        self.status= truecryptResponse.LISTPATTERN

    def already_mapped(self, *args):
        self.status= truecryptResponse.ALREADY_MAPPED
        return self.close(args)
    
    def cannot_open_volume(self, *args):
        self.status= truecryptResponse.CANNOT_OPEN_VOLUME
        return self.close(*args)
    
    def illegal_seek(self, *args):
        self.status= truecryptResponse.ILLEGAL_SEEK
        return self.close(*args)
    
    def mountpoint_does_not_exist(self, *args):
        self.status= truecryptResponse.MOUNT_POINT_DOES_NOT_EXIST
        return self.close(*args)
    
    def incorrect_volume(self, *args):
        self.status= truecryptResponse.INCORRECT_VOLUME
        return self.close(*args)
    
    def wrong_fs_type(self, *args):
        self.status= truecryptResponse.WRONG_FS_TYPE
        return self.close(*args)
    
################################################################################
# BASICS
################################################################################
    
    def eof(self, *args):
        if self.status != truecryptResponse.DO_NOT_OVERWRITE:
            self.status= truecryptResponse.EOF
        return self.close(*args)
     
    def close(self, *args):
        return True
    
    def not_a_directory(self, *args):
        self.status= NOT_A_DIRECTORY
        return self.close(*args)
################################################################################
# MOUSEMOVEMENT    
################################################################################
    def create_progress(self,callback, size, size_unit, *args):
        "callback: has .update(percent) and .set_text(string) methods"
        after= args[0]['child'].after
        match= truecryptResponse.CREATE_PROGRESS_REGEXP.search(after)
        size_done= match.group('size')
        size_done_unit= match.group('size_unit')
        
        if size_done_unit == "GB":
            size_done_unit= size_done_unit*1024
        elif size_done_unit == "KB":
            size_done_unit= size_done_unit/1024
        elif size_done_unit == "B":
            size_done_unit= (size_done_unit/1024)/1024
        
        if size_unit == "G":
            size= size*1024
        elif size_unit == "K":
            size= size/1024
        elif size_unit == "B":
            size= (size/1024)/1024
        
        if eval(size)-eval(size_done) < 4:
            percent= 1.0
        else:
            percent= eval(size_done) / eval(size)
        
        callback(percent, after)
        return
        
    def volume_created(self, *args):
        self.status= truecryptResponse.VOLUME_CREATED
        return self.close(*args)
    
    def mouse_connected(self,mouse_callback, *args):
        #mouse_callback.run()
        return "n\n"
    
    def mouse_movement(self, callback, *args):
        "gain random data by mouse movement"
        
        match= truecryptResponse.PROGRESS_REGEXP.search(args[0]['child'].after)
        percent = match.group('percent').lstrip(' ').rstrip('%')
        if not callback.update(percent):
            return self.close(*args)
        return "---\n"