#
# Revelation 0.4.7 - a password manager for GNOME 2
# http://oss.codepoet.no/revelation/
# $Id: util.py 476 2006-02-02 19:28:56Z erikg $
#
# Module with various utility functions
#

# Copyright (c) 2003-2006 Erik Grinaker
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import crack, math, string

def check_password(password):
	"Checks if a password is valid"

	# check for length
	if len(password) < 20:
		raise ValueError, "is too short"


	# check for entropy
	pwlen	= len(password)
	ent	= entropy(password)
	idealent = entropy_ideal(pwlen)

	if (pwlen < 100 and ent / idealent < 0.8) or (pwlen >= 100 and ent < 5.3):
		raise ValueError, "isn't varied enough"

	# check the password strength
	lc, uc, d, o = 0, 0, 0, 0

	for c in password:
		if c in string.ascii_lowercase:
			lc += 1

		elif c in string.ascii_uppercase:
			uc += 1

		elif c in string.digits:
			d += 1

		else:
			o += 1

	classcount = [ lc, uc, d, o ]
	classcount.sort()
	classcount.reverse()

	cred = sum([ count * (1 + (weight ** 2.161 / 10)) for weight, count in zip(range(1, len(classcount) + 1), classcount) ])

	if cred < 10:
		raise ValueError, "is too weak"


	# check if the password is a palindrome
	for i in range(len(password)):
		if password[i] != password[-i - 1]:
			break

	else:
		raise ValueError, "is a palindrome"


	# check password with cracklib
	try:
		if len(password) < 100:
			crack.FascistCheck(password)

	except ValueError, reason:

		# modify reason
		reason = str(reason).strip()
		reason = reason.replace("simplistic/systematic", "systematic")
		reason = reason.replace(" dictionary", "")

		if reason[:3] == "it ":
			reason = reason[3:]

		if reason[:5] == "it's ":
			reason = "is " + reason[5:]

		raise ValueError, reason

	except IOError:
		pass


def entropy(string):
	"Calculates the Shannon entropy of a string"

	# get probability of chars in string
	prob = [ float(string.count(c)) / len(string) for c in dict.fromkeys(list(string)) ]

	# calculate the entropy
	entropy = - sum([ p * math.log(p) / math.log(2.0) for p in prob ])

	return entropy


def entropy_ideal(length):
	"Calculates the ideal Shannon entropy of a string with given length"

	prob = 1.0 / length

	return -1.0 * length * prob * math.log(prob) / math.log(2.0)


