import dbus, texpect, truecryptResponse, mtpython, config, uiutil, os

class Volume:
    bus = dbus.SystemBus()
    hal_manager_obj = bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
    hal_manager = dbus.Interface(hal_manager_obj, 'org.freedesktop.Hal.Manager')
    conf= config.Config()
    truecrypt= texpect.Texpect()
    sudo_callback= uiutil.get_gksu_password
    options= {'user_mount'    : False,
              'read_only'     : False,
              'protect_hidden': False,
              'update_time'   : False,
              'filesystem'    : 'auto',
              'mount_options' : ''
              }
    
    def __init__(self, volume_path=None, mount_point=None, mapper_device=None, fromHistory= False, udi= None, options=None):
        if volume_path== None:
            try:
                devobj= Volume.bus.get_object('org.freedesktop.Hal', udi)
                self.volume_path= devobj.GetProperty('block.device')
            except dbus.exceptions.DBusException:
                print "Dbus Exception, nothing to worry about. Exception uses print by itself..."
                self.volume_path= None
        else:
            self.volume_path= volume_path
        
        if udi== None:
            self.udi= self.get_udi_of_volume()
            if self.udi == False:
                self.udi = self.volume_path
        else:
            self.udi= udi
            
        self.mapper_device= mapper_device
        self.mount_point= mount_point
        
        try:
            self.isFile= self.isFileVolume()
        except AttributeError:
            self.isFile= False
            
        if options != None: 
            self.set_options(options, False)
            
        self.fromHistory= fromHistory
    
    def isFileVolume(self):
        return not(self.volume_path.startswith("/dev") or self.volume_path.startswith("/dev/dm") or self.volume_path.startswith("/org/f"))
    
    def fixUserRights(self):
        tc = texpect.Texpect(config.DEBUG)
        c = "chown "+`os.getuid()`+":"+`os.getgid()`+" "+self.volume_path
        result= tc.doSudoCommand(c,
                 uiutil.get_gksu_password,
                 _("Get sudo password to fix access rights for "+self.volume_path)
                 )
            
        if result== truecryptResponse.TRY_AGAIN:
            popup.Popup(_("Wrong password"))
            return False
        return True
    
    def isMounted(self):
        volume_to_mountpoint = mtpython.FileSystemInfo(mounted_only=1).get(mount_point=self.mount_point)
        if [] == volume_to_mountpoint:
            return False
        pwp = texpect.Texpect(config.DEBUG)
        tvolumes = pwp.parsevolumelist(uiutil.get_gksu_password,
                                       _("Enter sudo password to check if volume is mounted")
                                       )
        
        if tvolumes == truecryptResponse.TRY_AGAIN:
            return truecryptResponse.TRY_AGAIN

        elif tvolumes == truecryptResponse.NO_VOLUMES_MAPPED:
            return False
        
        for mapper in tvolumes:
            if self.volume_path == mapper[1]:
                return True
        return False
    
    def isMapped(self):
        if self.mapper_device != None:
            return True
        else:
            return False
    
    def isPresent(self):
        if self.volume_path != None:
            return True
        else:
            return False
    
    def hasHistory(self):
        return self.fromHistory

    
    def mount(self, mount_path, protected, keyfiles, password, hidden_password=None, hidden_keyfiles=None):
        Volume.truecrypt= texpect.Texpect()
        message= _("Enter sudo password to mount volume")
        status= Volume.truecrypt.mount(self.volume_path, mount_path, self.options['protect_hidden'], keyfiles, password, self.options['filesystem'], self.options['mount_options'], self.options['read_only'], self.options['user_mount'], self.options['update_time'], sudo_callback=uiutil.get_gksu_password, message=message, hidden_password=hidden_password, hidden_keyfiles=hidden_keyfiles)
        if status== truecryptResponse.MOUNTING_SUCCESSFULL:
            self.isMounted= True
            self.mount_point= mount_path
        return status
    
    def map(self, protected, keyfiles, password):
        status= self.mount("", protected, keyfiles, password)
        if status== truecryptResponse.MOUNTING_SUCCESSFULL or status== truecryptResponse.ALREADY_MAPPED or status == truecryptResponse.VOLUME_MOUNTED:
            self.isMounted= False
            self.mount_point= None
            pwp = texpect.Texpect(config.DEBUG)
            tvolumes = pwp.parsevolumelist(uiutil.get_gksu_password,
                                       _("Enter sudo password to get mapper device")
                                       )
        
            if tvolumes == truecryptResponse.TRY_AGAIN:
                return truecryptResponse.TRY_AGAIN
            for mapper in tvolumes:
                if self.volume_path == mapper[1]:
                    self.mapper_device= mapper[0]
            
        return status

        
    def format(self, filesystem, protected, keyfiles, password):
        if self.isMounted():
            self.unmount()
        if not self.isMapped():
            self.map(protected, keyfiles, password)
            #TODO: evaluate status
            #TODO: integrate in dialogCreate
        
        c= "mkfs -t "+filesystem+" "+self.mapper_device
        tc= texpect.Texpect()
        result= tc.doSudoCommand(c,
                         uiutil.get_gksu_password,
                         _("Get sudo password to map volume "+self.volume_path)
                         )
            
        if result== truecryptResponse.TRY_AGAIN:
            popup.Popup(_("Wrong password"))
        elif result == False:
            return False
        self.unmount()
        return True
        
    def mount_mapped(self, mount_point):
        Volume.truecrypt= texpect.Texpect()
        message= _("Enter sudo password to mount mapped volume")
        status= Volume.truecrypt.mount_mapped(self, mount_point, uiutil.get_gksu_password, message)
        return status
    
    def unmount(self):
        Volume.truecrypt= texpect.Texpect()
        message= _("Enter sudo password to unmount volume ")
        if self.mount_point!= None:
            message= message+self.mount_point
            status= Volume.truecrypt.dismount(self.mount_point, uiutil.get_gksu_password, message)
        else:
            message= message+self.mapper_device
            status= Volume.truecrypt.dismount(self.mapper_device, uiutil.get_gksu_password, message)
        
        if status== truecryptResponse.DISMOUNTING_SUCCESSFULL:
            self.isMounted== False
        return status

    
    def properties(self, password, keyfiles):
       Volume.truecrypt= texpect.Texpect()
       message= _("Enter sudo password to get properties")
       return Volume.truecrypt.properties(self.volume_path, password, keyfiles, uiutil.get_gksu_password, message)

    def set_options(self, options, addToGconf= True):
       "options is the following map:     "
       "options= {                        "
       "       'user_mount'    : False,   "
       "       'read_only'     : False,   "
       "       'protect_hidden'    : False,   "
       "       'update_time' : True,   "
       "       'filesystem'    : '',    # this is the filesystem as described in man mount"
       "       'mount_options' : ''     # mount options as described in man mount"
       "       }                          "
       options['mount_options'].lstrip().rstrip()
       options['filesystem'].lstrip().rstrip()
       self.options= options
       if addToGconf== True:
           self.conf.add_volume_to_gconf(self)
       return self
       

    def get_udi_of_volume(self):
        if self.volume_path.startswith("/dev"):
            try:
                vpath= Volume.hal_manager.FindDeviceStringMatch('block.device', self.volume_path)[0]
            except:
                vpath= False
            return vpath
        else:
            return self.volume_path
    
    @classmethod
    def create(cls, volume, password, size, size_unit, encryption, hash, type,
               filesystem, keyfiles=None, cluster_size=None, display_keys=None,
               quick_format= None, mouse_callback=None, exists_callback=None,
               key_callback=None, progress_callback=None,):
        
        message= _("Enter sudo password to create volume")
        Volume.truecrypt= texpect.Texpect()
        status= Volume.truecrypt.create(
               volume, password, size, size_unit, encryption, hash, type,
               filesystem, keyfiles, cluster_size, display_keys, quick_format,
               uiutil.get_gksu_password, message, mouse_callback, exists_callback,
               key_callback, progress_callback)
        
       
        return status, Volume(volume)

    def get_haldev_for_volume(self):
        try:
            udi = Volume.hal_manager.FindDeviceStringMatch('block.device', self.volume_path)[0]
        except:
            return None # if it is a path
        devobj= Volume.bus.get_object('org.freedesktop.Hal', udi)
        if devobj.QueryCapability('volume'):
            return devobj
        else:
            return False
    
    def compareTo(self, anotherVolume):
        try:
            if (self.udi        == anotherVolume.udi# and
            #self.volume_path    == anotherVolume.volume_path and
            #self.mapper_device  == anotherVolume.mapper_device and
            #self.mount_point    == anotherVolume.mount_point and
            #self.isMounted      == anotherVolume.isMounted
            ):
                return True
            else:
                return False
        except AttributeError:
            return False
    def add_to_gconf(self):
        self.conf.add_volume_to_gconf(self)
    
    def is_a_truecrypt_device(self):
        devobj = Volume.bus.get_object('org.freedesktop.Hal', self.udi)
        try:
            size        = devobj.GetProperty('volume.size')
            num_blocks  = devobj.GetProperty('volume.num_blocks') 
            dev         = devobj.GetProperty ('block.device')
            isVolume    = devobj.QueryCapability("volume")
            hasFS       = devobj.GetProperty('volume.fstype')!=""
        except:
            dev = None
            return False
        
        if hasFS:
            "Return immediately: so no password is asked if truecrypt requires\
            superuser. Otherwise it gets the mounted volumes list from truecrypt\
            and then checks if the device added is already mounted"
            return False
        
        if size <= config.EXTENDED_LIMIT_SIZE and num_blocks <= config.EXTENDED_LIMIT_BLOCKS:
            return False
        
        elif isVolume and hasFS:
            return False
        
        elif isVolume and not hasFS:
            return True
    