import mtpython, volume, texpect, config, truecryptResponse

class Volumes:
    
    def __init__(self, sudo_callback=None):
        self.conf= config.Config()
        self.sudo_callback=sudo_callback
        self.volume_list= []
        self.addHistory()
    
    def addHistory(self):
        history= self.conf.get_history()
        if history == None:
            return
        
        for h in history:
            if not(h[0].startswith("/dev/mapper") or h[0].startswith("/dev/dm") or h[0].startswith("/org/f")):
                self.add_volume(volume.Volume(h[0], h[1], mapper_device= None, fromHistory= True, udi=None, options=h[2]))
            else:
                self.add_volume(volume.Volume(None, h[1], mapper_device= None, fromHistory= True, udi=h[0], options=h[2]))
    
    def add_volume(self, volume_to_add):
        for vol in self.volume_list:
            if vol.compareTo(volume_to_add):
                if vol.mount_point== None and volume_to_add.mount_point != None:
                    vol.mount_point= volume_to_add.mount_point
                return False

        self.volume_list.append(volume_to_add)
        volume.Volume.sudo_callback= self.sudo_callback
        return True
    
    def add_volumes(self, volume_list):
        for volume in volume_list:
            self.add_volume(volume)
    
    def remove_volume(self, volume):
        for vol in self.volume_list:
            if vol.vompareTo(volume):
                self.volume_list.remove(vol)
    
    def num_volumes_mapped(self):
        return len(self.volume_list)
    
    def add_volume_to_gconf(self, volume):
        self.conf.add_volume_to_gconf(volume)

    
    def delete_all_volumes_from_gconf(self):
        self.conf.clear_mount_history()
    
    def refresh(self):
        self.volume_list= []
        
        pwp = texpect.Texpect(config.DEBUG)
        tvolumes = pwp.parsevolumelist(self.sudo_callback,
                                       _("Enter sudo password to refresh view of truecrypt volumes")
                                       )
        if tvolumes == truecryptResponse.TRY_AGAIN:
            result= tvolumes            
            tvolumes = []

            
        elif tvolumes == truecryptResponse.NO_VOLUMES_MAPPED:
            result= tvolumes
            tvolumes = []
        else:
            result= True
 
        mapper_to_volume = tvolumes
        volume_to_mountpoint = mtpython.FileSystemInfo(mounted_only=1).get_fuzzy(spec='/dev/mapper/truecrypt')
        volume_to_mountpoint += mtpython.FileSystemInfo(mounted_only=1).get_fuzzy(spec='/dev/dm-')

        for mapper in mapper_to_volume:
            v= None
            for volume_m in volume_to_mountpoint:
                if mapper[0] == volume_m.spec or mapper[0][-1] == volume_m.spec[-1]:
                    v= volume.Volume(mapper[1], volume_m.file, mapper[0], fromHistory=False)
                    
            if v== None:
                v= volume.Volume(mapper[1], mapper_device=mapper[0], fromHistory= False)
            self.add_volume(v)
            
        self.addHistory()
        return self, result
        
    def by_udi(self, udi):
        for v in self.volume_list:
            try:
                if v.udi == udi:
                    return v
            except AttributeError:
                if config.DEBUG: print "Attribute Error"
        
        return False
    
    def by_mount_point(self, mp):
        for v in self.volume_list:
            try:
                if v.mount_point == mp:
                    return v
            except AttributeError:
                if config.DEBUG: print "Attribute Error"
        
        return False
    
    def by_volume_path(self, volume_path):
        for v in self.volume_list:
            try:
                if v.volume_path == volume_path:
                    return v
            except AttributeError:
                if config.DEBUG: print "Attribute Error"
        return False
    
    def by_mapper_device(self, mapper_device):
        for v in self.volume_list:
            try:
                if v.mapper_device == mapper_device:
                    return v
            except AttributeError:
                if config.DEBUG: print "Attribute Error"
        return False
    
    def by_status(self, mounted_or_not):
        hit_list= []
        for v in self.volume_list:
            try:
                if v.isMounted == mounted_or_not:
                    hit_list+= v
            except AttributeError:
                if config.DEBUG: print "Attribute Error"
        return hit_list
    
    def get_all_mounteds_volumes(self):
        mounted_volumes= []
        for volume in self.volume_list:
            if volume.isMounted():
                mounted_volumes.append(volume)
        return mounted_volumes
    
    def get_all_mapped_volumes(self):
        mounted_volumes= []
        for volume in self.volume_list:
            if volume.isMapped():
                mounted_volumes.append(volume)
        return mounted_volumes
    
    def get_saved_volumes(self):
        saved_volumes= []
        
        for volume in self.volume_list:
            if volume.hasHistory():
                saved_volumes.append(volume)                
        return saved_volumes
    
    def get_saved_file_volumes(self):
        saved_volumes= []
        
        for volume in self.volume_list:
            if volume.hasHistory() and volume.isFile:
                saved_volumes.append(volume)
                
        return saved_volumes
    
    def get_saved_device_volumes(self):
        saved_volumes= []
        
        for volume in self.volume_list:
            if volume.hasHistory() and not volume.isFile:
                saved_volumes.append(volume)
                
        return saved_volumes
    
    def get_saved_and_present_device_volumes(self):
        "and not mounted... btw. a filter would be nice for this"
        saved_volumes= []
        
        for volume in self.volume_list:
            if volume.hasHistory() and not volume.isFile and volume.isPresent() and not volume.isMounted():
                saved_volumes.append(volume)
                
        return saved_volumes    
    
    def dismount_all(self):
        pwp = texpect.Texpect()
        return pwp.dismount(None, self.sudo_callback, _("Enter sudo password to dismount all volumes"))
    
    def get_all_connected_volumes(self):
        udi_list = self.hal_manager.FindDeviceStringMatch('info.category', 'volume')
        return udi_list
    
    def remove_volume_from_history(self, udi):
        vol = self.by_udi(udi)
        try:
            self.volume_list.remove(vol)
            self.conf.remove_volume_from_gconf(vol)
            return True
        except:
            return False